#!/bin/sh
# ============================================================================
#
# This file is part of the 'FFP-stick for ZyXEL NSA-nnn'
# http://zyxel.nas-central.org/wiki/FFP-stick
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#
# This script is supposed to download, unpack and start ffp
# First argument /path/to/ffproot
# Second argument Nastype (NSA210,NSA220,...)
# Third argument firmware version

SCRIPTVERSION="20110323"
FFPROOT=$1
NAS=$2
FIRMWARE=$3
# Change this value to <> 0 to attempt not to chroot
DONTCHROOT=0
CHROOTDIR=${FFPROOT}

CleanUp()
{
    echo Delete some DNS-323 specific stuff...
    for file in start/passwd.sh start/rcS.sh start/shells.sh sbin/store-passwd.sh
    do
	rm -f ffp/${file}
    done
    echo Done
}

Unpack_fun_plug()
{
    echo Unpack ${FFPROOT}/$1...
    cd ffp
    tar -xzf ../$1
    cd ..
    rm $1
    echo Done
    CleanUp
}

CheckForFfpTarball()
{
    # find ffp tarball
    for file in fun_plug.tgz ../fun_plug.tgz
    do
	if [ -r ${file} ]; then
	    Unpack_fun_plug ${file}
	fi
    done
}

TryToDownload()
{
    if ! [ -x wget ]; then
	echo No wget found
	return 0
    fi

    echo Will try to download fun_plug.tgz
    echo If the script stops here, downloading the tarball from inreto.de failed.
    echo You can try to get internet access for the NAS,
    echo "or copy fun_plug.tgz (case sensitive!) to the share containing ffproot."
    echo It should be recognized and installed.
    echo It is possible that you have to reboot the box before this share shows up.
    echo ping www.inreto.de...
    while ! [ -r ffp/bin/busybox ] && ! ping -c 1 www.inreto.de >/dev/null ; do
	sleep 10
	CheckForFfpTarball
    done  

    if ! [ -r ffp/bin/busybox ]; then
	echo www.inreto.de found. Downloading fun_plug.tgz...
	cd ffp
	../wget -qO- http://www.inreto.de/dns323/fun-plug/0.5/fun_plug.tgz | tar -xz
	if [ "$?" != "0" ] ; then
    	    echo Download failed.
	else
	    echo fun_plug.tgz is downloaded
	    CleanUp
	fi
    	cd ..
    fi
    
    # Say hello to the author of this script. This doesn't do anything, (the file doesn't exist, so you'll get
    # a 404), except generating a logline on my webserver. (When it's online), Then I know that yet another 
    # FFP-stick is created. I like that. ;)
    # If you want to opt-out, just remove the line below
    ./wget -qO- http://mijzelf.hopto.org/hello_author.php?nas=${NAS}\&firmware=${FIRMWARE} >/dev/null 2>&1
    rm ./wget
}

#####################
# Function added 20101217 to solve the problem that the /i-data/md0
# symlink was not copied right on the NSA-320
#####################
CopySymlink()
{
    echo CopySymlink $1 $2
    target=` ls -l $1 | awk -F\  '{print $NF}' `    
    echo ln -s $target $2
    ln -s $target $2
}

CopyTree() 
{
    echo CopyTree $1 $2 $3
    
    if [ -h $2 ] ; then
	# Directory turns out to be a symlink. Just copy it
	CopySymlink $2 $3
	return 0
    fi
    
    mkdir $3
    if mountpoint $2 ; then
	# Directory is a mountpoint
	mount --bind $2 $3
	if [ "0" != "$?" ] ; then
	    return 1
	fi
	return 0
    fi 
	
    for file in ` ls $2 `
    do
	if [ -d $2/${file} ] ; then
	# File is a directory
	    CopyTree $1 $2/${file} $3/${file}
	    if [ "0" != "$?" ] ; then
	        return 1
	    fi
	#########################################
	# Added 2011-03-08; Changed 2011-03-23;
	# The NSA-320 has a symlink /etc/exports, 
	# which points to a non-existing file
	# as long as the nfs packet is not installed.
	# This frustrated this copy script
	elif [ -h $2/${file} ] ; then
	    # File is a symlink. Just copy it
	    CopySymlink $2/${file} $3/${file}
	#########################################
	else
	    case $1 in
		ln)
		    ln $2/${file} $3/${file}
		    ;;
		cp)
		    cp -p $2/${file} $3/${file}
		    ;;
	    esac
	    if [ "0" != "$?" ] ; then
		return 1
	    fi 
	fi
    done
    return 0
}
	     

NSA320BindAll()
{
    chdir=/tmp/bindroot
    
    mkdir ${chdir}
    for dir in bin sbin lib usr i-data e-data dev mnt
    do
	CopyTree ln /${dir} ${chdir}/${dir}
	if [ "0" != "$?" ]; then
	    echo Failed to CopyTree /${dir}
	    return 1
	fi
    done

    mkdir ${chdir}/tmp
    mount -t tmpfs tmfs ${chdir}/tmp
    if [ "0" != "$?" ]; then
	echo Failed to mount tmpfs on ${chdir}/tmp
        return 1
    fi

    echo ` date ` Waiting for fadd to show up...
    while ! ps | grep /sbin/fadd >/dev/null ; do
	sleep 10
    done
    echo ` date ` Hello fadd!

    CopyTree cp /etc ${chdir}/tmp/.etc
    if [ "0" != "$?" ]; then
	echo Failed to CopyTree /etc
        return 1
    fi

    # Because the files in /etc are bound to be changed by the firmware, we can't just make hardlinks to
    # it. We'll just hijack the map...
    mount --bind ${chdir}/tmp/.etc /etc
    if [ "0" != "$?" ]; then
	echo Failed to bind ${chdir}/tmp/.etc on /etc
        return 1
    fi
    
    ##################
    # Added 20101221 to solve the problem that the 'default' shares were not 
    # exported, only the user made ones. The mount mtd4 on /etc/zyxel was lost.
    echo mount --bind ${chdir}/tmp/.etc/zyxel /etc/zyxel
    mount --bind ${chdir}/tmp/.etc/zyxel /etc/zyxel
    ##################
    
    ln -s ${FFPROOT}/ffp ${chdir}/ffp
    ln -s ${FFPROOT}/ffp/etc ${chdir}/etc
    ln -s ${FFPROOT}/after_booting.sh ${chdir}/after_booting.sh
    
    mount --bind ${chdir}/tmp/.etc ${chdir}${FFPROOT}/ffp/etc/original
    mount --bind ${chdir}/tmp/.etc/zyxel ${chdir}${FFPROOT}/ffp/etc/zyxel

    # We'll hijack /var/log to, for the same reason...
    ln -s ${FFPROOT}/var ${chdir}/var
    CopyTree cp /var/log ${chdir}/tmp/.log
    mount --bind ${chdir}/tmp/.log /var/log
    mount --bind ${chdir}/tmp/.log ${chdir}${FFPROOT}/var/log

    if ! [ -d ${chdir}/dev/pts ]; then 
	mkdir ${chdir}/dev/pts
    fi

    mount -t devpts devpts ${chdir}/dev/pts
    mkdir ${chdir}/proc
    mount -t proc proc ${chdir}/proc
    mkdir ${chdir}/sys
    mount -t sysfs sys ${chdir}/sys

    mkdir ${chdir}/home
    mount --bind ${FFPROOT}/home ${chdir}/home

    # Restart Samba because it has gone mad about the etc exchange
    /etc/init.d/samba.sh restart

    CHROOTDIR=${chdir}
    return 0
}

ClassicBindAll()
{
    # mount directories for chroot
    tomount="dev var tmp bin sbin lib usr"
    for arg in bind rbind
    do
	for tm in ${tomount}
	do
	    mount --${arg} /${tm} ${tm}
	    if [ "$?" != "0" ] ; then
		return 1
	    fi
	done
	tomount="i-data e-data"
    done

    mount --bind /etc ffp/etc/original
    mount --bind /etc/zyxel ffp/etc/zyxel
    mount -t proc proc proc
    mount -t sysfs sys sys
    CHROOTDIR=${FFPROOT}
    return 0
}

FinalBindAll()
{
    CopyTree cp /dev ${FFPROOT}/dev
    if ! [ -d ${FFPROOT}/dev/pts ] ; then
        mkdir ${FFPROOT}/dev/pts
    fi
    mount -t devpts devpts ${FFPROOT}/dev/pts
    for dir in i-data e-data
    do
	mkdir ${FFPROOT}/${dir}
	CopyTree cp /${dir} ${FFPROOT}/${dir} 
    done
    mount -t proc proc proc
    mount -t sysfs sys sys
    CHROOTDIR=${FFPROOT}
}

StartFFP()
{
    # create rootdirectory
    if [ -r rootfs.tgz ]; then
	echo Extract rootfs.tgz...
        tar -xzf rootfs.tgz
	rm rootfs.tgz
	echo Done
    fi

    fun_plug=""
    
    CheckForFfpTarball
    
    if ! [ -r ffp/bin/busybox ] ; then
	TryToDownload
    fi

    if ! [ -r ffp/bin/busybox ] ; then
	echo Could not download fun_plug.tgz for some reason
	echo Will start polling for the existance forever in the share containing ffproot...
	while ! [ -r ffp/bin/busybox ]; do
	    sleep 10
	    CheckForFfpTarball
	done
    fi

    if [ "0" != "${DONTCHROOT}" ]; then
	echo Trying to create a symlink in the root
	ln -s ${FFPROOT}/ffp /ffp
	if ! [ -x /ffp/bin/busybox ]; then
	    echo Could not create symlink in root. Will continue to chroot
	fi
    fi

    if [ -x /ffp/bin/busybox ] ; then
	# Some braveheart created a symlink to ffproot/ffp
	# Will obey that...
	
	# Have a look if password have to be changed:
	if ! cat /etc/passwd | grep ^user ; then
	    echo Adding/changing passwords...
	    adduser -D user
	    for name in user root
	    do
		password=${name} # Change this line if you want a different password
		
		makepwd ${password} | sed -e 's/\//\\\//g' > /tmp/wkdhfwe0f9e9fujfkef
		password=`cat /tmp/wkdhfwe0f9e9fujfkef`
		rm /tmp/wkdhfwe0f9e9fujfkef
		
		cp /etc/shadow /tmp/shadow.bak
		cat /tmp/shadow.bak  | sed -e 's/^${name}\:\$.\$[^\:]*\:/{&name}:'${password}':/g'> /etc/shadow
		rm -f /tmp/shadow.bak
	    done
	fi

	# run fun_plug local, if present
	if [ -x /ffp/etc/fun_plug.local ]; then
	    /ffp/etc/fun_plug.local
	fi
 
	# start services
	if [ -x /ffp/etc/rc ]; then
	    /ffp/etc/rc
	fi
    
	exit 0
    fi

    ClassicBindAll
    if [ "$?" != "0" ]; then
    	echo "Bummer! Cannot bind directories. Will try the NSA320 hack to get it going"
	NSA320BindAll
        if [ "$?" != "0" ]; then
	    echo "The NSA320 hack doesn't work either. Will do the best I can to get it working"
	    echo Possibilities are limited.
	    FinalBindAll
	fi
    fi

    echo Will chroot ffp on ${CHROOTDIR}
    
    # run fun_plug local, if present
    if [ -x ${CHROOTDIR}/ffp/etc/fun_plug.local ]; then
	chroot ${CHROOTDIR} /ffp/etc/fun_plug.local
    fi
 
    # start services
    if [ -x ${CHROOTDIR}/ffp/etc/rc ]; then
	chroot ${CHROOTDIR} /ffp/etc/rc
    fi
    
    exit 0
}

echo Starting $0 version ${SCRIPTVERSION} at ` date `...
StartFFP

